@@ -1,4 +1,4 @@
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -9,9 +9,10 @@ from .optimization import optimizerlib as optimizers  # busy namespace, likely t
 from .optimization import families as families
 from .optimization import callbacks as callbacks
 from .common import errors as errors
+from . import ops as ops
 
 
-__all__ = ["optimizers", "families", "callbacks", "p", "typing", "errors"]
+__all__ = ["optimizers", "families", "callbacks", "p", "typing", "errors", "ops"]
 
 
-__version__ = "0.4.3.post2"
+__version__ = "1.0.4"
