@@ -1,4 +1,4 @@
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -10,12 +10,14 @@ import hashlib
 import warnings
 import argparse
 import itertools
+from collections import defaultdict
 from pathlib import Path
 import numpy as np
 import pandas as pd
 from matplotlib import pyplot as plt
 from matplotlib.legend import Legend
-from matplotlib import cm
+
+# from matplotlib import cm
 from mpl_toolkits.axes_grid1 import make_axes_locatable
 import nevergrad.common.typing as tp
 from . import utils
@@ -25,11 +27,21 @@ from .exporttable import export_table
 
 
 _DPI = 250
-
+pure_algorithms = []
 
 # %% Basic tools
 
 
+def compactize(name: str) -> str:
+    if len(name) < 70:
+        return name
+    hashcode = hashlib.md5(bytes(name, "utf8")).hexdigest()
+    name = re.sub(r"\([^()]*\)", "", name)
+    mid = 35
+    name = name[:mid] + hashcode + name[-mid:]
+    return name
+
+
 def _make_style_generator() -> tp.Iterator[str]:
     lines = itertools.cycle(["-", "--", ":", "-."])  # 4
     markers = itertools.cycle("ov^<>8sp*hHDd")  # 13
@@ -88,7 +100,16 @@ def aggregate_winners(
         return aggregate_winners(df, categories[1:], all_optimizers)
     iterdf, iternum = zip(
         *(
-            aggregate_winners(df.loc[df.loc[:, categories[0]] == val], categories[1:], all_optimizers)
+            aggregate_winners(
+                df.loc[
+                    df.loc[:, categories[0]]
+                    == val
+                    # if categories[0] != "budget"
+                    # else df.loc[:, categories[0]] <= val
+                ],
+                categories[1:],
+                all_optimizers,
+            )
             for val in subcases
         )
     )
@@ -153,7 +174,7 @@ _PARAM_MERGE_PATTERN = "{optimizer_name},{parametrization}"
 
 
 def merge_optimizer_name_pattern(
-    df: utils.Selector, pattern: str, merge_parametrization: bool = False
+    df: utils.Selector, pattern: str, merge_parametrization: bool = False, remove_suffix: bool = False
 ) -> utils.Selector:
     """Merge the optimizer name with other descriptors based on a pattern
     Nothing happens if merge_parametrization is false and pattern is empty string
@@ -182,6 +203,8 @@ def merge_optimizer_name_pattern(
             inds = sub.loc[:, okey] == optim
             if len(sub.loc[inds, :].unique(others)) > 1:
                 df.loc[inds, okey] = sub.loc[inds, elements].agg(aggregate, axis=1)
+    if remove_suffix:
+        df["optimizer_name"] = df["optimizer_name"].replace(r"[0-9\.\-]*$", "", regex=True)
     return df.drop(columns=others)  # type: ignore
 
 
@@ -207,6 +230,7 @@ def create_plots(
     max_combsize: int = 1,
     xpaxis: str = "budget",
     competencemaps: bool = False,
+    nomanyxp: bool = False,
 ) -> None:
     """Saves all representing plots to the provided folder
 
@@ -224,6 +248,63 @@ def create_plots(
     assert xpaxis in ["budget", "pseudotime"]
     df = remove_errors(df)
     df.loc[:, "loss"] = pd.to_numeric(df.loc[:, "loss"])
+    df = df.loc[:, [x for x in df.columns if not x.startswith("info/")]]
+    # Normalization of types.
+    for col in df.columns:
+        print(" Working on ", col)
+        failed_indices: tp.List[tp.Any] = []
+        if "max_irr" in col:
+            df[col] = df[col].round(decimals=4)
+        if col in (
+            "budget",
+            "num_workers",
+            "dimension",
+            "useful_dimensions",
+            "num_blocks",
+            "block_dimension",
+            "num_objectives",
+            "loss",
+        ):
+            try:
+                df[col] = (
+                    df[col].astype(float).astype(int)
+                    if ("num" in col or "dim" in col or "budget" in col)
+                    else df[col].astype(float)
+                )
+                # print(col, " is converted to int")
+            except Exception as e1:
+                for i in range(len(df[col])):
+                    try:
+                        float(df[col][i])
+                    except Exception as e2:
+                        if len(failed_indices) < 20:
+                            print("Error ", e2)
+                        failed_indices += [i]
+                assert (
+                    len(failed_indices) < 500
+                ), f"Fails at row {i+2}, Exceptions: {e1}. Failed-indices = {failed_indices}"
+                for i0 in range(len(failed_indices)):
+                    i = failed_indices[len(failed_indices) - 1 - i0]
+                    df.drop(index=i, inplace=True)
+                    print("We drop index ", i, "/", len(df), " for ", col)
+                try:
+                    df[col] = (
+                        df[col].astype(float).astype(int)
+                        if ("num" in col or "dim" in col or "budget" in col)
+                        else df[col].astype(float)
+                    )
+                except:
+                    print(f"Failed for {col}")
+
+        elif col != "loss":
+            df[col] = df[col].astype(str)
+            df[col] = df[col].replace(r"\.[0]*$", "", regex=True)
+            try:
+                df.loc[:, col] = pd.to_numeric(df.loc[:, col])
+            except:
+                pass
+    if "num_objectives" in df.columns:
+        df = df[df.num_objectives != 0]  # the optimization did not even start
     # If we have a descriptor "instrum_str",
     # we assume that it describes the instrumentation as a string,
     # that we should include the various instrumentations as distinct curves in the same plot.
@@ -235,6 +316,8 @@ def create_plots(
         df = df.drop(columns="dimension")
         if "parametrization" in set(df.columns):
             df = df.drop(columns="parametrization")
+        if "instrumentation" in set(df.columns):
+            df = df.drop(columns="instrumentation")
     df = utils.Selector(df.fillna("N-A"))  # remove NaN in non score values
     assert not any("Unnamed: " in x for x in df.columns), f"Remove the unnamed index column:  {df.columns}"
     assert "error " not in df.columns, f"Remove error rows before plotting"
@@ -311,7 +394,9 @@ def create_plots(
         for case in df.unique(fixed) if fixed else [()]:
             print("\n# new case #", fixed, case)
             casedf = df.select(**dict(zip(fixed, case)))
-            data_df = FightPlotter.winrates_from_selection(casedf, fight_descriptors, num_rows=num_rows)
+            data_df = FightPlotter.winrates_from_selection(
+                casedf, fight_descriptors, num_rows=num_rows, num_cols=350
+            )
             fplotter = FightPlotter(data_df)
             # Competence maps: we find out the best algorithm for each attribute1=valuei/attribute2=valuej.
             if order == 2 and competencemaps and best_algo:
@@ -320,19 +405,33 @@ def create_plots(
             # save
             name = "fight_" + ",".join("{}{}".format(x, y) for x, y in zip(fixed, case)) + ".png"
             name = "fight_all.png" if name == "fight_.png" else name
+            name = compactize(name)
+            fullname = name
 
-            if name == "fight_all.png":
-                with open(str(output_folder / name) + ".cp.txt", "w") as f:
-                    f.write("ranking:\n")
-                    for i, algo in enumerate(data_df.columns[:8]):
-                        f.write(f"  algo {i}: {algo}\n")
             if len(name) > 240:
                 hashcode = hashlib.md5(bytes(name, "utf8")).hexdigest()
                 name = re.sub(r"\([^()]*\)", "", name)
                 mid = 120
                 name = name[:mid] + hashcode + name[-mid:]
             fplotter.save(str(output_folder / name), dpi=_DPI)
-
+            # Second version, restricted to cases with all data available.
+            data_df = FightPlotter.winrates_from_selection(
+                casedf, fight_descriptors, num_rows=num_rows, complete_runs_only=True
+            )
+            fplotter = FightPlotter(data_df)
+            if name == "fight_all.png":
+                with open(str(output_folder / name) + ".cp.txt", "w") as f:
+                    f.write(fullname)
+                    f.write("ranking:\n")
+                    global pure_algorithms
+                    pure_algorithms = list(data_df.columns[:])
+                    for i, algo in enumerate(data_df.columns[:158]):
+                        f.write(f"  algo {i}: {algo}\n")
+            if name == "fight_all.png":
+                fplotter.save(str(output_folder / "fight_all_pure.png"), dpi=_DPI)
+            else:
+                fplotter.save(str(output_folder / name) + "_pure.png", dpi=_DPI)
+                print(f"# {len(data_df.columns[:])}  {data_df.columns[:]}")
             if order == 2 and competencemaps and best_algo:  # With order 2 we can create a competence map.
                 print("\n# Competence map")
                 name = "competencemap_" + ",".join("{}".format(x) for x in fixed) + ".tex"
@@ -350,18 +449,27 @@ def create_plots(
     # Average normalized plot with everything.
     out_filepath = output_folder / "xpresults_all.png"
     data = XpPlotter.make_data(df, normalized_loss=True)
-    xpplotter = XpPlotter(data, title=os.path.basename(output_folder), name_style=name_style, xaxis=xpaxis)
+    xpplotter = XpPlotter(
+        data, title=os.path.basename(output_folder), name_style=name_style, xaxis=xpaxis, pure_only=True
+    )
     xpplotter.save(out_filepath)
     # Now one xp plot per case.
     for case in cases:
+        if nomanyxp:
+            continue
         subdf = df.select_and_drop(**dict(zip(descriptors, case)))
         description = ",".join("{}:{}".format(x, y) for x, y in zip(descriptors, case))
+        full_description = description
+        description = compactize(description)
         if len(description) > 280:
             hash_ = hashlib.md5(bytes(description, "utf8")).hexdigest()
             description = description[:140] + hash_ + description[-140:]
         out_filepath = output_folder / "xpresults{}{}.png".format(
             "_" if description else "", description.replace(":", "")
         )
+        txt_out_filepath = output_folder / "xpresults{}{}.leaderboard.txt".format(
+            "_" if description else "", description.replace(":", "")
+        )
         data = XpPlotter.make_data(subdf)
         try:
             xpplotter = XpPlotter(data, title=description, name_style=name_style, xaxis=xpaxis)
@@ -369,9 +477,25 @@ def create_plots(
             warnings.warn(f"Bypassing error in xpplotter:\n{e}", RuntimeWarning)
         else:
             xpplotter.save(out_filepath)
+            xpplotter.save_txt(txt_out_filepath, data, full_description)
     plt.close("all")
 
 
+def gp_sota() -> tp.Dict[str, tp.Tuple[float, float]]:
+    gp = {}
+    gp["CartPole-v1"] = (-500.0, 100000.0)
+    gp["Acrobot-v1"] = (83.17, 200000.0)
+    gp["MountainCarContinuous-v0"] = (-99.31, 900000.0)
+    gp["Pendulum-v0"] = (154.36, 1100000.0)
+    gp["InvertedPendulumSwingupBulletEnv-v0"] = (-893.35, 400000.0)
+    gp["BipedalWalker-v3"] = (-268.85, 1100000.0)
+    gp["BipedalWalkerHardcore-v3"] = (-9.25, 1100000.0)
+    gp["HopperBulletEnv-v0"] = (-999.19, 1000000.0)
+    gp["InvertedDoublePendulumBulletEnv-v0"] = (-9092.17, 300000.0)
+    gp["LunarLanderContinuous-v2"] = (-287.58, 1000000.0)
+    return gp
+
+
 class LegendInfo(tp.NamedTuple):
     """Handle for information used to create a legend."""
 
@@ -403,6 +527,7 @@ class XpPlotter:
         title: str,
         name_style: tp.Optional[tp.Dict[str, tp.Any]] = None,
         xaxis: str = "budget",
+        pure_only: bool = False,
     ) -> None:
         if name_style is None:
             name_style = NameStyle()
@@ -415,11 +540,23 @@ class XpPlotter:
         # plot from best to worst
         lowerbound = np.inf
         sorted_optimizers = sorted(optim_vals, key=lambda x: optim_vals[x]["loss"][-1], reverse=True)
+        if pure_only:
+            assert len(pure_algorithms) > 0
+            sorted_optimizers = [
+                o for o in sorted_optimizers if o + " " in [p[: (len(o) + 1)] for p in pure_algorithms]
+            ]
+            with open("rnk__" + str(title) + ".cp.txt", "w") as f:
+                f.write(compactize(title))
+                f.write("ranking:\n")
+                for i, algo in reversed(list(enumerate(sorted_optimizers))):
+                    f.write(f"  algo {i}: {algo} (x)\n")
+            # print(sorted_optimizers, " merged with ", pure_algorithms)
+            # print("Leads to ", sorted_optimizers)
         self._fig = plt.figure()
         self._ax = self._fig.add_subplot(111)
         # use log plot? yes, if no negative value
         logplot = not any(
-            x <= 0 or x > 10 ** 8 for ov in optim_vals.values() for x in ov["loss"]
+            x <= 0 or x > 10**8 for ov in optim_vals.values() for x in ov["loss"]
         )  # if x < np.inf)
         if logplot:
             self._ax.set_yscale("log")
@@ -434,14 +571,49 @@ class XpPlotter:
         self._ax.grid(True, which="both")
         self._overlays: tp.List[tp.Any] = []
         legend_infos: tp.List[LegendInfo] = []
-        for optim_name in sorted_optimizers:
+        title_addendum = f"({len(sorted_optimizers)} algos)"
+        for optim_name in (
+            sorted_optimizers[:1] + sorted_optimizers[-35:]
+            if len(sorted_optimizers) > 35
+            else sorted_optimizers
+        ):
             vals = optim_vals[optim_name]
+            indices = np.where(vals["num_eval"] > 0)
             lowerbound = min(lowerbound, np.min(vals["loss"]))
+            # We here add some state of the art results.
+            # This adds a cross on figures, x-axis = budget and y-axis = loss.
+            for sota_name, sota in [("GP", gp_sota())]:
+                for k in sota.keys():
+                    if k in title:
+                        th = sota[k][0]  # loss of proposed solution.
+                        cost = sota[k][1]  # Computational cost for the proposed result.
+                        title_addendum = f"({sota_name}:{th})"
+                        lowerbound = min(lowerbound, th, 0.9 * th, 1.1 * th)
+                        plt.plot(  # Horizontal line at the obtained GP cost.
+                            vals[xaxis][indices],
+                            th + 0 * vals["loss"][indices],
+                            name_style[optim_name],
+                            label="gp",
+                        )
+                        plt.plot(  # Vertical line, showing the budget of the GP solution.
+                            [cost] * 3,
+                            [
+                                min(vals["loss"][indices]),
+                                sum(vals["loss"][indices]) / len(indices),
+                                max(vals["loss"][indices]),
+                            ],
+                            name_style[optim_name],
+                            label="gp",
+                        )
             line = plt.plot(vals[xaxis], vals["loss"], name_style[optim_name], label=optim_name)
             # confidence lines
             for conf in self._get_confidence_arrays(vals, log=logplot):
                 plt.plot(vals[xaxis], conf, name_style[optim_name], label=optim_name, alpha=0.1)
-            text = "{} ({:.3g})".format(optim_name, vals["loss"][-1])
+            text = "{} ({:.3g} <{:.3g}>)".format(
+                optim_name,
+                vals["loss"][-1],
+                vals["loss"][-2] if len(vals["loss"]) > 2 else float("nan"),
+            )
             if vals[xaxis].size:
                 legend_infos.append(LegendInfo(vals[xaxis][-1], vals["loss"][-1], line, text))
         if not (np.isnan(upperbound) or np.isinf(upperbound)):
@@ -455,10 +627,15 @@ class XpPlotter:
                     )
             self._ax.set_ylim(top=upperbound_up)
         all_x = [v for vals in optim_vals.values() for v in vals[xaxis]]
-        self._ax.set_xlim([min(all_x), max(all_x)])
+        try:
+            all_x = [float(a_) for a_ in all_x]
+            self._ax.set_xlim([min(all_x), max(all_x)])  # type: ignore
+        except TypeError:
+            print(f"TypeError for minimum or maximum or {all_x}")
         self.add_legends(legend_infos)
         # global info
-        self._ax.set_title(split_long_title(title))
+        if "tmp" not in title:
+            self._ax.set_title(split_long_title(title + title_addendum))
         self._ax.tick_params(axis="both", which="both")
         # self._fig.tight_layout()
 
@@ -467,7 +644,7 @@ class XpPlotter:
         vals: tp.Dict[str, np.ndarray], log: bool = False
     ) -> tp.Tuple[np.ndarray, np.ndarray]:
         loss = vals["loss"]
-        conf = vals["loss_std"] / np.sqrt(vals["num_eval"] - 1)
+        conf = vals["loss_std"] / np.sqrt(vals["loss_nums"] - 1)
         if not log:
             return loss - conf, loss + conf
         lloss = np.log10(loss)
@@ -538,6 +715,7 @@ class XpPlotter:
         groupeddf = df.groupby(["optimizer_name", "budget"])
         means = groupeddf.mean()
         stds = groupeddf.std()
+        nums = groupeddf.count()
         optim_vals: tp.Dict[str, tp.Dict[str, np.ndarray]] = {}
         # extract name and coordinates
         for optim in df.unique("optimizer_name"):
@@ -545,12 +723,38 @@ class XpPlotter:
             optim_vals[optim]["budget"] = np.array(means.loc[optim, :].index)
             optim_vals[optim]["loss"] = np.array(means.loc[optim, "loss"])
             optim_vals[optim]["loss_std"] = np.array(stds.loc[optim, "loss"])
+            optim_vals[optim]["loss_nums"] = np.array(nums.loc[optim, "loss"])
             num_eval = np.array(groupeddf.count().loc[optim, "loss"])
             optim_vals[optim]["num_eval"] = num_eval
             if "pseudotime" in means.columns:
                 optim_vals[optim]["pseudotime"] = np.array(means.loc[optim, "pseudotime"])
         return optim_vals
 
+    @staticmethod
+    def save_txt(
+        output_filepath: tp.PathLike, optim_vals: tp.Dict[str, tp.Dict[str, np.ndarray]], addendum: str = ""
+    ) -> None:
+        """Saves a list of best performances.
+
+        output_filepath: Path or str
+            path where the figure must be saved
+        optim_vals: dict
+            dict of losses obtained by a given optimizer.
+        """
+        best_performance: tp.Dict[int, tp.Any] = defaultdict(lambda: (float("inf"), "none"))
+        for optim in optim_vals.keys():
+            for i, l in zip(optim_vals[optim]["budget"], optim_vals[optim]["loss"]):
+                if l < best_performance[i][0]:
+                    best_performance[i] = (l, optim)
+
+        with open(output_filepath, "w") as f:
+            f.write(addendum)
+            f.write("Best performance:\n")
+            for i in best_performance.keys():
+                f.write(
+                    f"  budget {i}: {best_performance[i][0]} ({best_performance[i][1]}) ({output_filepath})\n"
+                )
+
     def save(self, output_filepath: tp.PathLike) -> None:
         """Saves the xp plot
 
@@ -559,9 +763,15 @@ class XpPlotter:
         output_filepath: Path or str
             path where the figure must be saved
         """
-        self._fig.savefig(
-            str(output_filepath), bbox_extra_artists=self._overlays, bbox_inches="tight", dpi=_DPI
-        )
+        try:  # Let us catch errors due to too many DPIs.
+            self._fig.savefig(
+                str(output_filepath), bbox_extra_artists=self._overlays, bbox_inches="tight", dpi=_DPI
+            )
+        except ValueError as v:
+            print(f"We catch {v} which means that image = too big.")
+            self._fig.savefig(
+                str(output_filepath), bbox_extra_artists=self._overlays, bbox_inches="tight", dpi=_DPI / 5
+            )
 
     def __del__(self) -> None:
         plt.close(self._fig)
@@ -603,12 +813,17 @@ class FightPlotter:
         self.winrates = winrates_df
         self._fig = plt.figure()
         self._ax = self._fig.add_subplot(111)
+        max_cols = 25
         self._cax = self._ax.imshow(
-            100 * np.array(self.winrates), cmap=cm.seismic, interpolation="none", vmin=0, vmax=100
+            100 * np.array(self.winrates)[:, :max_cols],
+            cmap="seismic",
+            interpolation="none",
+            vmin=0,
+            vmax=100,
         )
-        x_names = self.winrates.columns
+        x_names = self.winrates.columns[:max_cols]  # we plot only the 50 best
         self._ax.set_xticks(list(range(len(x_names))))
-        self._ax.set_xticklabels(x_names, rotation=90, fontsize=7)  # , ha="left")
+        self._ax.set_xticklabels(x_names, rotation=45, ha="right", fontsize=7)
         y_names = self.winrates.index
         self._ax.set_yticks(list(range(len(y_names))))
         self._ax.set_yticklabels(y_names, rotation=45, fontsize=7)
@@ -620,7 +835,11 @@ class FightPlotter:
 
     @staticmethod
     def winrates_from_selection(
-        df: utils.Selector, categories: tp.List[str], num_rows: int = 5
+        df: utils.Selector,
+        categories: tp.List[str],
+        num_rows: int = 5,
+        num_cols: int = 350,
+        complete_runs_only: bool = False,
     ) -> pd.DataFrame:
         """Creates a fight plot win rate data out of the given run dataframe,
         by iterating over all cases with fixed category variables.
@@ -633,11 +852,19 @@ class FightPlotter:
             List of variables to fix for obtaining similar run conditions
         num_rows: int
             number of rows to plot (best algorithms)
+        complete_runs_only: bool
+            if we want a plot with only algorithms which have run on all settings
         """
         all_optimizers = list(df.unique("optimizer_name"))  # optimizers for which no run exists are not shown
         num_rows = min(num_rows, len(all_optimizers))
         # iterate on all sub cases
         victories, total = aggregate_winners(df, categories, all_optimizers)
+        if complete_runs_only:
+            max_num = max([int(2 * victories.loc[n, n]) for n in all_optimizers])
+            new_all_optimizers = [n for n in all_optimizers if int(2 * victories.loc[n, n]) == max_num]
+            if len(new_all_optimizers) > 0:
+                df = df[df["optimizer_name"].isin(new_all_optimizers)]
+                victories, total = aggregate_winners(df, categories, new_all_optimizers)
         # subcases = df.unique(categories)
         # for k, subcase in enumerate(subcases):  # TODO linearize this (precompute all subcases)? requires memory
         #     # print(subcase)
@@ -651,12 +878,15 @@ class FightPlotter:
         sorted_names = winrates.index
         # number of subcases actually computed is twice self-victories
         sorted_names = ["{} ({}/{})".format(n, int(2 * victories.loc[n, n]), total) for n in sorted_names]
-        sorted_names = [sorted_names[i] for i in range(min(30, len(sorted_names)))]
+        num_names = len(sorted_names)
+        sorted_names = [sorted_names[i] for i in range(min(num_cols, num_names))]
         data = np.array(winrates.iloc[:num_rows, : len(sorted_names)])
         # pylint: disable=anomalous-backslash-in-string
         best_names = [
-            (f"{name} ({100 * val:2.1f}%)").replace("Search", "")
-            for name, val in zip(mean_win.index[:num_rows], mean_win)
+            (
+                f"{name} ({i+1}/{num_names}:{100 * val:2.1f}% +- {25 * np.sqrt(val*(1-val)/int(2 * victories.loc[name, name])):2.1f})"
+            ).replace("Search", "")
+            for i, (name, val) in enumerate(zip(mean_win.index[:num_rows], mean_win))
         ]
         return pd.DataFrame(index=best_names, columns=sorted_names, data=data)
 
@@ -735,7 +965,9 @@ def compute_best_placements(positions: tp.List[float], min_diff: float) -> tp.Li
     ----
     This function is probably not optimal, but seems a very good heuristic
     """
-    assert all(v2 >= v1 for v2, v1 in zip(positions[1:], positions[:-1]))
+    assert all(
+        v2 >= v1 for v2, v1 in zip(positions[1:], positions[:-1])
+    )  # , str(zip(LegendGroup, positions))
     groups = [LegendGroup([k], [pos], min_diff) for k, pos in enumerate(positions)]
     new_groups: tp.List[LegendGroup] = []
     ready = False
@@ -784,11 +1016,22 @@ def main() -> None:
     parser.add_argument(
         "--competencemaps", type=bool, default=False, help="whether we should export only competence maps"
     )
+    parser.add_argument(
+        "--nomanyxp",
+        type=bool,
+        default=False,
+        help="whether we should remove the export of detailed convergence curves",
+    )
     parser.add_argument(
         "--merge-parametrization",
         action="store_true",
         help="if present, parametrization is merge into the optimizer name",
     )
+    parser.add_argument(
+        "--remove-suffix",
+        action="store_true",
+        help="if present, remove numerical suffixes in fight plots",
+    )
     parser.add_argument(
         "--merge-pattern",
         type=str,
@@ -798,10 +1041,12 @@ def main() -> None:
     )
     args = parser.parse_args()
     exp_df = merge_optimizer_name_pattern(
-        utils.Selector.read_csv(args.filepath), args.merge_pattern, args.merge_parametrization
+        utils.Selector.read_csv(args.filepath),
+        args.merge_pattern,
+        args.merge_parametrization,
+        args.remove_suffix,
     )
-    # merging names
-    #
+    exp_df.replace("CSEC11", "NGIohTuned", inplace=True)
     output_dir = args.output
     if output_dir is None:
         output_dir = str(Path(args.filepath).with_suffix("")) + "_plots"
@@ -811,6 +1056,7 @@ def main() -> None:
         max_combsize=args.max_combsize if not args.competencemaps else 2,
         xpaxis="pseudotime" if args.pseudotime else "budget",
         competencemaps=args.competencemaps,
+        nomanyxp=args.nomanyxp,
     )
 
 
