@@ -1,4 +1,4 @@
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -7,7 +7,7 @@ import numpy as np
 from . import core
 
 
-def test_mixsimulator() -> None:
+def test_fishing() -> None:
     np.random.seed(17)
     func = core.OptimizeFish()
     x = np.random.rand(func.dimension)
