@@ -1,4 +1,4 @@
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -90,7 +90,7 @@ class _Game:
         if policy1 is None and policy2 is None:
             return 57 * 57
         if np.random.uniform(0.0, 1.0) > 0.5:
-            r = self.flip_play_game_nosym(policy2, policy1)
+            r = self.flip_play_game_nosym(policy1=policy2, policy2=policy1)
             return 1 if r == 2 else 2 if r == 1 else 0
         return self.flip_play_game_nosym(policy1, policy2)
 
@@ -122,6 +122,9 @@ class _Game:
                 visible1, visible2, len(visible1) + len(cards1), len(visible2) + len(cards2), stack, policy1
             )
             we_play = False
+            next_visible1 = None
+            next_stack = None
+            next_visible2 = None
             for i in range(len(visible1)):  # pylint: disable=consider-using-enumerate
                 for location in range(2):
                     # print("testing ", visible1[i], " on ", stack[location])
@@ -145,8 +148,8 @@ class _Game:
                             we_play = True
             if we_play:
                 something_moves = True
-                visible1 = sorted(next_visible1 + ([cards1[0]] if cards1 else []))
-                stack = sorted(next_stack)
+                visible1 = sorted(next_visible1 + ([cards1[0]] if cards1 else []))  # type: ignore
+                stack = sorted(next_stack)  # type: ignore
                 if cards1:
                     del cards1[0]
                 if not visible1:
@@ -178,7 +181,7 @@ class _Game:
                             we_play = True
             if we_play:
                 something_moves = True
-                visible2 = sorted(next_visible2 + ([cards2[0]] if cards2 else []))
+                visible2 = sorted(next_visible2 + ([cards2[0]] if cards2 else []))  # type: ignore
                 stack = sorted(next_stack)
                 if cards2:
                     del cards2[0]
@@ -394,7 +397,7 @@ class _Game:
             seed = policy[a * 26 * 13 + b * 13 + c]  # type: ignore
         if seed == 0.0:
             return cards
-        state = np.random.RandomState(hash(seed) % (2 ** 32))
+        state = np.random.RandomState(hash(seed) % (2**32))
         state.shuffle(cards)
         return list(cards)
 
@@ -416,8 +419,8 @@ class Game(ExperimentFunction):
             self.game_object.play_game(self.game) * 2
         )  # times 2 because we consider both players separately.
         super().__init__(self._simulate_game, p.Array(shape=(dimension,)))
-        self.parametrization.descriptors.deterministic_function = False
-        self.parametrization.descriptors.metrizable = game not in ["war", "batawaf"]
+        self.parametrization.function.deterministic = False
+        self.parametrization.function.metrizable = game not in ["war", "batawaf"]
 
     def _simulate_game(self, x: np.ndarray) -> float:
         # FIXME: an adaptive opponent, e.g. bandit, would be better.
@@ -433,7 +436,7 @@ class Game(ExperimentFunction):
         return (result + (0.0 if r == 2 else 0.5 if r == 0 else 1.0)) / 2
 
     def evaluation_function(self, *recommendations: p.Parameter) -> float:
-        assert len(recommendations) == 1, "Should not be a pareto set for a monoobjective function"
+        assert len(recommendations) == 1, "Should not be a pareto set for a singleobjective function"
         x = recommendations[0].value
         # pylint: disable=not-callable
         loss = sum([self.function(x) for _ in range(42)]) / 42.0
