@@ -1,4 +1,4 @@
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -90,6 +90,7 @@ class Optimizer:  # pylint: disable=too-many-instance-attributes
         # you can also replace or reinitialize this random state
         self.num_workers = int(num_workers)
         self.budget = budget
+        self.optim_curve: tp.List[tp.Any] = []
 
         # How do we deal with cheap constraints i.e. constraints which are fast and use low resources and easy ?
         # True ==> we penalize them (infinite values for candidates which violate the constraint).
@@ -99,7 +100,7 @@ class Optimizer:  # pylint: disable=too-many-instance-attributes
 
         self.parametrization = (
             parametrization
-            if not isinstance(parametrization, (int, np.int))
+            if not isinstance(parametrization, (int, np.int_))
             else p.Array(shape=(parametrization,))
         )
         self.parametrization.freeze()  # avoids issues!
@@ -107,9 +108,9 @@ class Optimizer:  # pylint: disable=too-many-instance-attributes
             raise ValueError("No variable to optimize in this parametrization.")
         self.name = self.__class__.__name__  # printed name in repr
         # keep a record of evaluations, and current bests which are updated at each new evaluation
-        self.archive: utils.Archive[
-            utils.MultiValue
-        ] = utils.Archive()  # dict like structure taking np.ndarray as keys and Value as values
+        self.archive: utils.Archive[utils.MultiValue] = (
+            utils.Archive()
+        )  # dict like structure taking np.ndarray as keys and Value as values
         self.current_bests = {
             x: utils.MultiValue(self.parametrization, np.inf, reference=self.parametrization)
             for x in ["optimistic", "pessimistic", "average", "minimum"]
@@ -133,6 +134,18 @@ class Optimizer:  # pylint: disable=too-many-instance-attributes
         # to make optimize function stoppable halway through
         self._running_jobs: tp.List[tp.Tuple[p.Parameter, tp.JobLike[tp.Loss]]] = []
         self._finished_jobs: tp.Deque[tp.Tuple[p.Parameter, tp.JobLike[tp.Loss]]] = deque()
+        self._sent_warnings: tp.Set[tp.Any] = set()  # no use printing several time the same
+        # Most optimizers are designed for single objective and use a float loss.
+        # To use these in a multi-objective optimization, we provide the negative of
+        # the hypervolume of the pareto front as the loss.
+        # If not needed, an optimizer can set this to True.
+        self._no_hypervolume = False
+
+    def _warn(self, msg: str, e: tp.Any) -> None:
+        """Warns only once per warning type"""
+        if e not in self._sent_warnings:
+            warnings.warn(msg, e)
+            self._sent_warnings.add(e)
 
     @property
     def _rng(self) -> np.random.RandomState:
@@ -282,7 +295,13 @@ class Optimizer:  # pylint: disable=too-many-instance-attributes
         self._suggestions.append(self.parametrization.spawn_child(new_value=new_value))
 
     # pylint: disable=too-many-branches
-    def tell(self, candidate: p.Parameter, loss: tp.Loss) -> None:
+    def tell(
+        self,
+        candidate: p.Parameter,
+        loss: tp.Loss,
+        constraint_violation: tp.Optional[tp.Loss] = None,
+        penalty_style: tp.Optional[tp.ArrayLike] = None,
+    ) -> None:
         """Provides the optimizer with the evaluation of a fitness value for a candidate.
 
         Parameters
@@ -291,6 +310,13 @@ class Optimizer:  # pylint: disable=too-many-instance-attributes
             point where the function was evaluated
         loss: float/list/np.ndarray
             loss of the function (or multi-objective function
+        constraint_violation: float/list/np.ndarray/None
+            constraint violation (> 0 means that this is not correct)
+        penalty_style: ArrayLike/None
+            to be read as [a,b,c,d,e,f]
+            with cv the constraint violation vector (above):
+            penalty = (a + sum(|loss|)) * (f+num_tell)**e * (b * sum(cv**c)) ** d
+            default: [1e5, 1., .5, 1., .5, 1.]
 
         Note
         ----
@@ -306,23 +332,27 @@ class Optimizer:  # pylint: disable=too-many-instance-attributes
         will use this suggestion.
         """
         # Check loss type
-        if isinstance(loss, (Real, float)):
+        if isinstance(loss, (Real, float)) or (isinstance(loss, np.ndarray) and not loss.shape):
             # using "float" along "Real" because mypy does not understand "Real" for now Issue #3186
             loss = float(loss)
             # Non-sense values including NaNs should not be accepted.
             # We do not use max-float as various later transformations could lead to greater values.
             if not loss < 5.0e20:  # pylint: disable=unneeded-not
-                warnings.warn(
+                self._warn(
                     f"Clipping very high value {loss} in tell (rescale the cost function?).",
                     errors.LossTooLargeWarning,
                 )
                 loss = 5.0e20  # sys.float_info.max leads to numerical problems so let us do this.
         elif isinstance(loss, (tuple, list, np.ndarray)):
-            loss = np.array(loss, copy=False, dtype=float).ravel() if len(loss) != 1 else loss[0]
+            loss = np.asarray(loss, dtype=float).ravel() if len(loss) != 1 else loss[0]
         elif not isinstance(loss, np.ndarray):
             raise TypeError(
                 f'"tell" method only supports float values but the passed loss was: {loss} (type: {type(loss)}.'
             )
+        if isinstance(loss, float) and (
+            len(self.optim_curve) == 0 or self.num_tell > self.optim_curve[-1][0] * 1.1
+        ):
+            self.optim_curve += [(self.num_tell, loss)]
         # check Parameter
         if not isinstance(candidate, p.Parameter):
             raise TypeError(
@@ -342,7 +372,9 @@ class Optimizer:  # pylint: disable=too-many-instance-attributes
                 raise RuntimeError("MultiobjectiveReference can only be provided before the first tell.")
             if not isinstance(loss, np.ndarray):
                 raise RuntimeError("MultiobjectiveReference must only be used for multiobjective losses")
-            self._hypervolume_pareto = mobj.HypervolumePareto(upper_bounds=loss, seed=self._rng)
+            self._hypervolume_pareto = mobj.HypervolumePareto(
+                upper_bounds=loss, seed=self._rng, no_hypervolume=self._no_hypervolume
+            )
             if candidate.value is None:
                 return  # no value, so stopping processing there
             candidate = candidate.value
@@ -358,11 +390,28 @@ class Optimizer:  # pylint: disable=too-many-instance-attributes
             # multiobjective reference is not handled :s
             # but this allows obtaining both scalar and multiobjective loss (through losses)
             callback(self, candidate, loss)
-        if not candidate.satisfies_constraints() and self.budget is not None:
+        if not candidate.satisfies_constraints(self.parametrization) and self.budget is not None:
             penalty = self._constraints_manager.penalty(candidate, self.num_ask, self.budget)
             loss = loss + penalty
-        if isinstance(loss, float):
+
+        if constraint_violation is not None:
+            if penalty_style is not None:
+                a, b, c, d, e, f = penalty_style
+            else:
+                a, b, c, d, e, f = (1e5, 1.0, 0.5, 1.0, 0.5, 1.0)
+
+            violation = float(
+                (a + np.sum(np.maximum(loss, 0.0)))
+                * ((f + self._num_tell) ** e)
+                * (b * np.sum(np.maximum(constraint_violation, 0.0) ** c) ** d)
+            )
+            loss += violation
+
+        if isinstance(loss, float) and (
+            self.num_objectives == 1 or self.num_objectives > 1 and not self._no_hypervolume
+        ):
             self._update_archive_and_bests(candidate, loss)
+
         if candidate.uid in self._asked:
             self._internal_tell_candidate(candidate, loss)
             self._asked.remove(candidate.uid)
@@ -373,7 +422,9 @@ class Optimizer:  # pylint: disable=too-many-instance-attributes
 
     def _preprocess_multiobjective(self, candidate: p.Parameter) -> tp.FloatLoss:
         if self._hypervolume_pareto is None:
-            self._hypervolume_pareto = mobj.HypervolumePareto(auto_bound=self._MULTIOBJECTIVE_AUTO_BOUND)
+            self._hypervolume_pareto = mobj.HypervolumePareto(
+                auto_bound=self._MULTIOBJECTIVE_AUTO_BOUND, no_hypervolume=self._no_hypervolume
+            )
         return self._hypervolume_pareto.add(candidate)
 
     def _update_archive_and_bests(self, candidate: p.Parameter, loss: tp.FloatLoss) -> None:
@@ -385,7 +436,7 @@ class Optimizer:  # pylint: disable=too-many-instance-attributes
                 f'"tell" method only supports float values but the passed loss was: {loss} (type: {type(loss)}.'
             )
         if np.isnan(loss) or loss == np.inf:
-            warnings.warn(f"Updating fitness with {loss} value", errors.BadLossWarning)
+            self._warn(f"Updating fitness with {loss} value", errors.BadLossWarning)
         mvalue: tp.Optional[utils.MultiValue] = None
         if x not in self.archive:
             self.archive[x] = utils.MultiValue(candidate, loss, reference=self.parametrization)
@@ -441,26 +492,33 @@ class Optimizer:  # pylint: disable=too-many-instance-attributes
         # - no memory of previous iterations.
         # - just projection to constraint satisfaction.
         # We try using the normal tool during half constraint budget, in order to reduce the impact on the normal run.
+        self.parametrization.tabu_fails = 0
         for _ in range(max_trials):
             is_suggestion = False
             if self._suggestions:  # use suggestions if available
                 is_suggestion = True
                 candidate = self._suggestions.pop()
             else:
-                candidate = self._internal_ask_candidate()
-            if candidate.satisfies_constraints():
+                try:  # Sometimes we have a limited budget so that
+                    candidate = self._internal_ask_candidate()
+                except AssertionError as e:
+                    assert (
+                        self.parametrization._constraint_checkers
+                    ), f"Error: {e}"  # This should not happen without constraint issues.
+                    candidate = self.parametrization.spawn_child()
+            if candidate.satisfies_constraints(self.parametrization):
                 break  # good to go!
-            if self._penalize_cheap_violations:
+            if self._penalize_cheap_violations or self.no_parallelization:
                 # Warning! This might be a tell not asked.
                 self._internal_tell_candidate(candidate, float("Inf"))  # DE requires a tell
             # updating num_ask  is necessary for some algorithms which need new num to ask another point
             self._num_ask += 1
-        satisfies = candidate.satisfies_constraints()
-        if not satisfies:
+        satisfies = candidate.satisfies_constraints(self.parametrization)
+        if not satisfies and self.parametrization.tabu_length == 0:
             # still not solving, let's run sub-optimization
             candidate = _constraint_solver(candidate, budget=max_trials)
-        if not (satisfies or candidate.satisfies_constraints()):
-            warnings.warn(
+        if not (satisfies or candidate.satisfies_constraints(self.parametrization, no_tabu=True)):
+            self._warn(
                 f"Could not bypass the constraint after {max_trials} tentatives, "
                 "sending candidate anyway.",
                 errors.FailedConstraintWarning,
@@ -501,6 +559,10 @@ class Optimizer:  # pylint: disable=too-many-instance-attributes
             The candidate with minimal loss. :code:`p.Parameters` have field :code:`args` and :code:`kwargs` which can be directly used
             on the function (:code:`objective_function(*candidate.args, **candidate.kwargs)`).
         """
+        if self.num_objectives > 1:
+            raise RuntimeError(
+                "No best candidate in MOO. Use optimizer.pareto_front() instead to get the set of all non-dominated candidates."
+            )
         recom_data = self._internal_provide_recommendation()  # pylint: disable=assignment-from-none
         if recom_data is None or any(np.isnan(recom_data)):
             name = "minimum" if self.parametrization.function.deterministic else "pessimistic"
@@ -535,12 +597,24 @@ class Optimizer:  # pylint: disable=too-many-instance-attributes
         """Override to provide a recommendation in standardized space"""
         return None
 
+    def enable_pickling(self) -> None:
+        """
+        Some optimizers are only optionally picklable, because picklability
+        requires saving the whole history which would be a waste of memory
+        in general. To tell an optimizer to be picklable, call this function
+        before any asks.
+
+        In this base class, the function is a no-op, but it is overridden
+        in some optimizers.
+        """
+
     def minimize(
         self,
         objective_function: tp.Callable[..., tp.Loss],
         executor: tp.Optional[tp.ExecutorLike] = None,
         batch_mode: bool = False,
         verbosity: int = 0,
+        constraint_violation: tp.Any = None,
     ) -> p.Parameter:
         """Optimization (minimization) procedure
 
@@ -552,13 +626,15 @@ class Optimizer:  # pylint: disable=too-many-instance-attributes
             An executor object, with method :code:`submit(callable, *args, **kwargs)` and returning a Future-like object
             with methods :code:`done() -> bool` and :code:`result() -> float`. The executor role is to dispatch the execution of
             the jobs locally/on a cluster/with multithreading depending on the implementation.
-            Eg: :code:`concurrent.futures.ThreadPoolExecutor`
+            Eg: :code:`concurrent.futures.ProcessPoolExecutor`
         batch_mode: bool
             when :code:`num_workers = n > 1`, whether jobs are executed by batch (:code:`n` function evaluations are launched,
             we wait for all results and relaunch n evals) or not (whenever an evaluation is finished, we launch
             another one)
         verbosity: int
             print information about the optimization (0: None, 1: fitness values, 2: fitness values and recommendation)
+        constraint_violation: list of functions or None
+            each function in the list returns >0 for a violated constraint.
 
         Returns
         -------
@@ -576,7 +652,7 @@ class Optimizer:  # pylint: disable=too-many-instance-attributes
         if executor is None:
             executor = utils.SequentialExecutor()  # defaults to run everything locally and sequentially
             if self.num_workers > 1:
-                warnings.warn(
+                self._warn(
                     f"num_workers = {self.num_workers} > 1 is suboptimal when run sequentially",
                     errors.InefficientSettingsWarning,
                 )
@@ -600,7 +676,12 @@ class Optimizer:  # pylint: disable=too-many-instance-attributes
                 while self._finished_jobs:
                     x, job = self._finished_jobs[0]
                     result = job.result()
-                    self.tell(x, result)
+                    if constraint_violation is not None:
+                        self.tell(
+                            x, result, [f(x.value) for f in constraint_violation]
+                        )  # TODO: this is not parallelized, wtf!
+                    else:
+                        self.tell(x, result)
                     self._finished_jobs.popleft()  # remove it after the tell to make sure it was indeed "told" (in case of interruption)
                     if verbosity:
                         print(f"Updating fitness with value {job.result()}")
@@ -635,7 +716,11 @@ class Optimizer:  # pylint: disable=too-many-instance-attributes
                 (tmp_finished if x_job[1].done() else tmp_runnings).append(x_job)
             self._running_jobs, self._finished_jobs = tmp_runnings, tmp_finished
             first_iteration = False
-        return self.provide_recommendation()
+        return self.provide_recommendation() if self.num_objectives == 1 else p.Constant(None)
+
+    def _info(self) -> tp.Dict[str, tp.Any]:
+        """Easy access to debug/benchmark info"""
+        return {}
 
 
 # Adding a comparison-only functionality to an optimizer.
@@ -669,7 +754,8 @@ class ConfiguredOptimizer:
     Parameters
     ----------
     OptimizerClass: type
-        class of the optimizer to configure
+        class of the optimizer to configure, or another ConfiguredOptimizer (config will then be ignored
+        except for the optimizer name/representation)
     config: dict
         dictionnary of all the configurations
     as_config: bool
@@ -686,9 +772,7 @@ class ConfiguredOptimizer:
     one_shot = False  # algorithm designed to suggest all budget points at once
     no_parallelization = False  # algorithm which is designed to run sequentially only
 
-    def __init__(
-        self, OptimizerClass: tp.Type[Optimizer], config: tp.Dict[str, tp.Any], as_config: bool = False
-    ) -> None:
+    def __init__(self, OptimizerClass: OptCls, config: tp.Dict[str, tp.Any], as_config: bool = False) -> None:
         self._OptimizerClass = OptimizerClass
         config.pop("self", None)  # self comes from "locals()"
         config.pop("__class__", None)  # self comes from "locals()"
@@ -700,7 +784,11 @@ class ConfiguredOptimizer:
         if not as_config:
             # try instantiating for init checks
             # if as_config: check can be done before setting attributes
-            self(parametrization=4, budget=100)
+            with warnings.catch_warnings():
+                warnings.filterwarnings(
+                    "ignore", category=errors.InefficientSettingsWarning
+                )  # this check does not need to be efficient
+                self(parametrization=4, budget=100)
 
     def config(self) -> tp.Dict[str, tp.Any]:
         return dict(self._config)
@@ -719,7 +807,9 @@ class ConfiguredOptimizer:
         num_workers: int
             number of evaluations which will be run in parallel at once
         """
-        config = dict(config=self) if self._as_config else self._config
+        config = dict(config=self) if self._as_config else self.config()
+        if isinstance(self._OptimizerClass, ConfiguredOptimizer):
+            config = {}  # ignore, it's already configured
         run = self._OptimizerClass(parametrization=parametrization, budget=budget, num_workers=num_workers, **config)  # type: ignore
         run.name = self.name
         # hacky but convenient to have around:
@@ -751,6 +841,7 @@ def _constraint_solver(parameter: p.Parameter, budget: int) -> p.Parameter:
     """Runs a suboptimization to solve the parameter constraints"""
     parameter_without_constraint = parameter.copy()
     parameter_without_constraint._constraint_checkers.clear()
+    parameter_without_constraint.tabu_length = 0
     opt = registry["OnePlusOne"](parameter_without_constraint, num_workers=1, budget=budget)
     for _ in range(budget):
         cand = opt.ask()
