@@ -1,4 +1,4 @@
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -7,15 +7,25 @@ from .oneshot import SamplingSearch
 from .differentialevolution import DifferentialEvolution
 from .optimizerlib import RandomSearchMaker, SQP, LHSSearch, DE, RandomSearch, MetaRecentering, MetaTuneRecentering  # type: ignore
 from .optimizerlib import (
-    MultipleSingleRuns,
+    ParametrizedMetaModel,
     ParametrizedOnePlusOne,
     ParametrizedCMA,
-    ConfSplitOptimizer,
-    ParametrizedBO,
+    # ParametrizedBO,
     EMNA,
+    CmaFmin2,
     NGOpt10,
+    NGOpt12,
+    BayesOptim,
+    # ConfPortfolio,
+    # DiagonalCMA,
+    # GeneticDE,
+    # TBPSA,
+    # NoisyOnePlusOne,
+    # RecombiningPortfolioOptimisticNoisyDiscreteOnePlusOne,
+    # OptimisticNoisyOnePlusOne,
 )
-from .optimizerlib import CMA, Chaining, PSO, BO
+from . import optimizerlib as opts
+from .optimizerlib import CMA, Chaining, PSO  # , BO
 
 # DE
 OnePointDE = DifferentialEvolution(crossover="onepoint").set_name("OnePointDE", register=True)
@@ -38,6 +48,19 @@ MicroCMA = ParametrizedCMA(scale=1e-6).set_name("MicroCMA", register=True)
 FCMAs03 = ParametrizedCMA(fcmaes=True, scale=0.3).set_name("FCMAs03", register=True)
 FCMAp13 = ParametrizedCMA(fcmaes=True, scale=0.1, popsize=13).set_name("FCMAp13", register=True)
 ECMA = ParametrizedCMA(elitist=True).set_name("ECMA", register=True)
+MetaModelDiagonalCMA = ParametrizedMetaModel(multivariate_optimizer=ParametrizedCMA(diagonal=True)).set_name(
+    "MetaModelDiagonalCMA", register=True
+)
+MetaModelFmin2 = ParametrizedMetaModel(multivariate_optimizer=CmaFmin2).set_name(
+    "MetaModelFmin2", register=True
+)
+MetaModelFmin2.no_parallelization = True
+LSCMA = ParametrizedCMA(high_speed=False).set_name("LSCMA", register=True)
+HSCMA = ParametrizedCMA(high_speed=True).set_name("HSCMA", register=True)
+HSNeuralCMA = ParametrizedCMA(high_speed=True, algorithm="neural").set_name("HSNeuralCMA", register=True)
+HSSVMCMA = ParametrizedCMA(high_speed=True, algorithm="svm").set_name("HSSVMCMA", register=True)
+HSRFCMA = ParametrizedCMA(high_speed=True, algorithm="rf").set_name("HSRFCMA", register=True)
+HSMetaModel = ParametrizedMetaModel(multivariate_optimizer=HSCMA).set_name("HSMetaModel", register=True)
 
 # OnePlusOne
 FastGADiscreteOnePlusOne = ParametrizedOnePlusOne(mutation="fastga").set_name(
@@ -46,6 +69,13 @@ FastGADiscreteOnePlusOne = ParametrizedOnePlusOne(mutation="fastga").set_name(
 DoubleFastGAOptimisticNoisyDiscreteOnePlusOne = ParametrizedOnePlusOne(
     noise_handling="optimistic", mutation="doublefastga"
 ).set_name("DoubleFastGAOptimisticNoisyDiscreteOnePlusOne", register=True)
+RecombiningGA = ParametrizedOnePlusOne(
+    mutation="doublefastga",
+    crossover=True,
+).set_name("RecombiningGA", register=True)
+RotatedRecombiningGA = ParametrizedOnePlusOne(
+    mutation="doublefastga", crossover=True, rotation=True
+).set_name("RotatedRecombiningGA", register=True)
 FastGAOptimisticNoisyDiscreteOnePlusOne = ParametrizedOnePlusOne(
     noise_handling="optimistic", mutation="fastga"
 ).set_name("FastGAOptimisticNoisyDiscreteOnePlusOne", register=True)
@@ -63,10 +93,10 @@ RecombiningOptimisticNoisyDiscreteOnePlusOne = ParametrizedOnePlusOne(
 ).set_name("RecombiningOptimisticNoisyDiscreteOnePlusOne", register=True)
 
 # BO
-RBO = ParametrizedBO(initialization="random").set_name("RBO", register=True)
-QRBO = ParametrizedBO(initialization="Hammersley").set_name("QRBO", register=True)
-MidQRBO = ParametrizedBO(initialization="Hammersley", middle_point=True).set_name("MidQRBO", register=True)
-LBO = ParametrizedBO(initialization="LHS").set_name("LBO", register=True)
+# RBO = ParametrizedBO(initialization="random").set_name("RBO", register=True)
+# QRBO = ParametrizedBO(initialization="Hammersley").set_name("QRBO", register=True)
+# MidQRBO = ParametrizedBO(initialization="Hammersley", middle_point=True).set_name("MidQRBO", register=True)
+# LBO = ParametrizedBO(initialization="LHS").set_name("LBO", register=True)
 
 # EMNA
 IsoEMNA = EMNA(naive=False).set_name("IsoEMNA", register=True)
@@ -108,18 +138,18 @@ ChainDEwithMetaRecenteringdim = Chaining([MetaRecentering, DE], ["dimension"]).s
 ChainDEwithMetaRecentering30 = Chaining([MetaRecentering, DE], [30]).set_name(
     "ChainDEwithMetaRecentering30", register=True
 )
-ChainBOwithMetaTuneRecentering = Chaining([MetaTuneRecentering, BO], ["num_workers"]).set_name(
-    "ChainBOwithMetaTuneRecentering", register=True
-)
-ChainBOwithMetaTuneRecenteringsqrt = Chaining([MetaTuneRecentering, BO], ["sqrt"]).set_name(
-    "ChainBOwithMetaTuneRecenteringsqrt", register=True
-)
-ChainBOwithMetaTuneRecenteringdim = Chaining([MetaTuneRecentering, BO], ["dimension"]).set_name(
-    "ChainBOwithMetaTuneRecenteringdim", register=True
-)
-ChainBOwithMetaTuneRecentering30 = Chaining([MetaTuneRecentering, BO], [30]).set_name(
-    "ChainBOwithMetaTuneRecentering30", register=True
-)
+# ChainBOwithMetaTuneRecentering = Chaining([MetaTuneRecentering, BO], ["num_workers"]).set_name(
+#    "ChainBOwithMetaTuneRecentering", register=True
+# )
+# ChainBOwithMetaTuneRecenteringsqrt = Chaining([MetaTuneRecentering, BO], ["sqrt"]).set_name(
+#    "ChainBOwithMetaTuneRecenteringsqrt", register=True
+# )
+# ChainBOwithMetaTuneRecenteringdim = Chaining([MetaTuneRecentering, BO], ["dimension"]).set_name(
+#    "ChainBOwithMetaTuneRecenteringdim", register=True
+# )
+# ChainBOwithMetaTuneRecentering30 = Chaining([MetaTuneRecentering, BO], [30]).set_name(
+#    "ChainBOwithMetaTuneRecentering30", register=True
+# )
 
 ChainDEwithMetaTuneRecentering = Chaining([MetaTuneRecentering, DE], ["num_workers"]).set_name(
     "ChainDEwithMetaTuneRecentering", register=True
@@ -135,27 +165,27 @@ ChainDEwithMetaTuneRecentering30 = Chaining([MetaTuneRecentering, DE], [30]).set
 )
 
 
-ChainBOwithR = Chaining([RandomSearch, BO], ["num_workers"]).set_name("ChainBOwithR", register=True)
-ChainBOwithRsqrt = Chaining([RandomSearch, BO], ["sqrt"]).set_name("ChainBOwithRsqrt", register=True)
-ChainBOwithRdim = Chaining([RandomSearch, BO], ["dimension"]).set_name("ChainBOwithRdim", register=True)
-ChainBOwithR30 = Chaining([RandomSearch, BO], [30]).set_name("ChainBOwithR30", register=True)
-ChainBOwithLHS30 = Chaining([LHSSearch, BO], [30]).set_name("ChainBOwithLHS30", register=True)
-ChainBOwithLHSsqrt = Chaining([LHSSearch, BO], ["sqrt"]).set_name("ChainBOwithLHSsqrt", register=True)
-ChainBOwithLHSdim = Chaining([LHSSearch, BO], ["dimension"]).set_name("ChainBOwithLHSdim", register=True)
-ChainBOwithLHS = Chaining([LHSSearch, BO], ["num_workers"]).set_name("ChainBOwithLHS", register=True)
-ChainBOwithMetaRecentering30 = Chaining([MetaRecentering, BO], [30]).set_name(
-    "ChainBOwithMetaRecentering30", register=True
-)
-ChainBOwithMetaRecenteringsqrt = Chaining([MetaRecentering, BO], ["sqrt"]).set_name(
-    "ChainBOwithMetaRecenteringsqrt", register=True
-)
-ChainBOwithMetaRecenteringdim = Chaining([MetaRecentering, BO], ["dimension"]).set_name(
-    "ChainBOwithMetaRecenteringdim", register=True
-)
-ChainBOwithMetaRecentering = Chaining([MetaRecentering, BO], ["num_workers"]).set_name(
-    "ChainBOwithMetaRecentering", register=True
-)
-
+# ChainBOwithR = Chaining([RandomSearch, BO], ["num_workers"]).set_name("ChainBOwithR", register=True)
+# ChainBOwithRsqrt = Chaining([RandomSearch, BO], ["sqrt"]).set_name("ChainBOwithRsqrt", register=True)
+# ChainBOwithRdim = Chaining([RandomSearch, BO], ["dimension"]).set_name("ChainBOwithRdim", register=True)
+# ChainBOwithR30 = Chaining([RandomSearch, BO], [30]).set_name("ChainBOwithR30", register=True)
+# ChainBOwithLHS30 = Chaining([LHSSearch, BO], [30]).set_name("ChainBOwithLHS30", register=True)
+# ChainBOwithLHSsqrt = Chaining([LHSSearch, BO], ["sqrt"]).set_name("ChainBOwithLHSsqrt", register=True)
+# ChainBOwithLHSdim = Chaining([LHSSearch, BO], ["dimension"]).set_name("ChainBOwithLHSdim", register=True)
+# ChainBOwithLHS = Chaining([LHSSearch, BO], ["num_workers"]).set_name("ChainBOwithLHS", register=True)
+# ChainBOwithMetaRecentering30 = Chaining([MetaRecentering, BO], [30]).set_name(
+#    "ChainBOwithMetaRecentering30", register=True
+# )
+# ChainBOwithMetaRecenteringsqrt = Chaining([MetaRecentering, BO], ["sqrt"]).set_name(
+#    "ChainBOwithMetaRecenteringsqrt", register=True
+# )
+# ChainBOwithMetaRecenteringdim = Chaining([MetaRecentering, BO], ["dimension"]).set_name(
+#    "ChainBOwithMetaRecenteringdim", register=True
+# )
+# ChainBOwithMetaRecentering = Chaining([MetaRecentering, BO], ["num_workers"]).set_name(
+#    "ChainBOwithMetaRecentering", register=True
+# )
+#
 ChainPSOwithR = Chaining([RandomSearch, PSO], ["num_workers"]).set_name("ChainPSOwithR", register=True)
 ChainPSOwithRsqrt = Chaining([RandomSearch, PSO], ["sqrt"]).set_name("ChainPSOwithRsqrt", register=True)
 ChainPSOwithRdim = Chaining([RandomSearch, PSO], ["dimension"]).set_name("ChainPSOwithRdim", register=True)
@@ -219,9 +249,9 @@ AvgHammersleySearch = SamplingSearch(sampler="Hammersley", recommendation_rule="
 AvgHammersleySearchPlusMiddlePoint = SamplingSearch(
     sampler="Hammersley", middle_point=True, recommendation_rule="average_of_best"
 ).set_name("AvgHammersleySearchPlusMiddlePoint", register=True)
-HCHAvgRandomSearch = SamplingSearch(sampler="Random", recommendation_rule="average_of_hull_best").set_name(
-    "HCHAvgRandomSearch", register=True
-)
+HullCenterHullAvgRandomSearch = SamplingSearch(
+    sampler="Random", recommendation_rule="average_of_hull_best"
+).set_name("HullCenterHullAvgRandomSearch", register=True)
 AvgRandomSearch = SamplingSearch(sampler="Random", recommendation_rule="average_of_best").set_name(
     "AvgRandomSearch", register=True
 )
@@ -247,37 +277,84 @@ TEAvgCauchyLHSSearch = SamplingSearch(
 ).set_name("TEAvgCauchyLHSSearch", register=True)
 
 # Recommendation rule = by convex hull.
-HCHAvgScrHaltonSearch = SamplingSearch(scrambled=True, recommendation_rule="average_of_hull_best").set_name(
-    "HCHAvgScrHaltonSearch", register=True
-)
-HCHAvgScrHaltonSearchPlusMiddlePoint = SamplingSearch(
+HullCenterHullAvgScrHaltonSearch = SamplingSearch(
+    scrambled=True, recommendation_rule="average_of_hull_best"
+).set_name("HullCenterHullAvgScrHaltonSearch", register=True)
+HullCenterHullAvgScrHaltonSearchPlusMiddlePoint = SamplingSearch(
     middle_point=True, scrambled=True, recommendation_rule="average_of_hull_best"
-).set_name("HCHAvgScrHaltonSearchPlusMiddlePoint", register=True)
-HCHAvgScrHammersleySearchPlusMiddlePoint = SamplingSearch(
+).set_name("HullCenterHullAvgScrHaltonSearchPlusMiddlePoint", register=True)
+HullCenterHullAvgScrHammersleySearchPlusMiddlePoint = SamplingSearch(
     scrambled=True, sampler="Hammersley", middle_point=True, recommendation_rule="average_of_hull_best"
-).set_name("HCHAvgScrHammersleySearchPlusMiddlePoint", register=True)
-HCHAvgLargeHammersleySearch = SamplingSearch(
+).set_name("HullCenterHullAvgScrHammersleySearchPlusMiddlePoint", register=True)
+HullCenterHullAvgLargeHammersleySearch = SamplingSearch(
     scale=100.0, sampler="Hammersley", recommendation_rule="average_of_hull_best"
-).set_name("HCHAvgLargeHammersleySearch", register=True)
-HCHAvgScrHammersleySearch = SamplingSearch(
+).set_name("HullCenterHullAvgLargeHammersleySearch", register=True)
+HullCenterHullAvgScrHammersleySearch = SamplingSearch(
     sampler="Hammersley", scrambled=True, recommendation_rule="average_of_hull_best"
-).set_name("HCHAvgScrHammersleySearch", register=True)
-HCHAvgCauchyScrHammersleySearch = SamplingSearch(
+).set_name("HullCenterHullAvgScrHammersleySearch", register=True)
+HullCenterHullAvgCauchyScrHammersleySearch = SamplingSearch(
     cauchy=True, sampler="Hammersley", scrambled=True, recommendation_rule="average_of_hull_best"
-).set_name("HCHAvgCauchyScrHammersleySearch", register=True)
-HCHAvgLHSSearch = SamplingSearch(sampler="LHS", recommendation_rule="average_of_hull_best").set_name(
-    "HCHAvgLHSSearch", register=True
-)
-HCHAvgCauchyLHSSearch = SamplingSearch(
+).set_name("HullCenterHullAvgCauchyScrHammersleySearch", register=True)
+HullCenterHullAvgLHSSearch = SamplingSearch(
+    sampler="LHS", recommendation_rule="average_of_hull_best"
+).set_name("HullCenterHullAvgLHSSearch", register=True)
+HullCenterHullAvgCauchyLHSSearch = SamplingSearch(
     sampler="LHS", cauchy=True, recommendation_rule="average_of_hull_best"
-).set_name("HCHAvgCauchyLHSSearch", register=True)
+).set_name("HullCenterHullAvgCauchyLHSSearch", register=True)
 
 # Split on top of competence map.
-MetaNGOpt10 = ConfSplitOptimizer(
+MetaNGOpt10 = opts.ConfSplitOptimizer(
     multivariate_optimizer=NGOpt10, monovariate_optimizer=NGOpt10, non_deterministic_descriptor=False
 ).set_name("MetaNGOpt10", register=True)
 
 # Multiple single runs for multi-objective optimization.
-NGOptSingle9 = MultipleSingleRuns(num_single_runs=9).set_name("NGOptSingle9", register=True)
-NGOptSingle16 = MultipleSingleRuns(num_single_runs=16).set_name("NGOptSingle16", register=True)
-NGOptSingle25 = MultipleSingleRuns(num_single_runs=25).set_name("NGOptSingle25", register=True)
+NGOptSingle9 = opts.MultipleSingleRuns(num_single_runs=9, base_optimizer=NGOpt12).set_name(
+    "NGOptSingle9", register=True
+)
+NGOptSingle16 = opts.MultipleSingleRuns(num_single_runs=16, base_optimizer=NGOpt12).set_name(
+    "NGOptSingle16", register=True
+)
+NGOptSingle25 = opts.MultipleSingleRuns(num_single_runs=25, base_optimizer=NGOpt12).set_name(
+    "NGOptSingle25", register=True
+)
+
+# noisy splitters
+Noisy13Splits = opts.NoisySplit(num_optims=13, discrete=False).set_name("Noisy13Splits", register=True)
+NoisyInfSplits = opts.NoisySplit(num_optims=float("inf"), discrete=False).set_name(
+    "NoisyInfSplits", register=True
+)
+DiscreteNoisy13Splits = opts.NoisySplit(num_optims=13, discrete=True).set_name(
+    "DiscreteNoisy13Splits", register=True
+)
+DiscreteNoisyInfSplits = opts.NoisySplit(num_optims=float("inf"), discrete=True).set_name(
+    "DiscreteNoisyInfSplits", register=True
+)
+
+# PCA-BO
+# Testing the influence of n_components on the performance of PCABO
+PCABO80 = BayesOptim(pca=True, n_components=0.80).set_name("PCABO80", register=True)
+
+# Testing the influence of the DoE size on the performance of PCABO
+PCABO95DoE20 = BayesOptim(pca=True, n_components=0.95, prop_doe_factor=0.20).set_name(
+    "PCABO95DoE20", register=True
+)
+SparseDiscreteOnePlusOne = ParametrizedOnePlusOne(mutation="discrete", sparse=True).set_name(
+    "SparseDiscreteOnePlusOne", register=True
+)
+
+# Specifically for RL.
+# MixDeterministicRL = ConfPortfolio(optimizers=[DiagonalCMA, PSO, GeneticDE]).set_name(
+#    "MixDeterministicRL", register=True
+# )
+# SpecialRL = Chaining([MixDeterministicRL, TBPSA], ["half"]).set_name("SpecialRL", register=True)
+# NoisyRL1 = Chaining([MixDeterministicRL, NoisyOnePlusOne], ["half"]).set_name("NoisyRL1", register=True)
+# NoisyRL2 = Chaining(
+#    [MixDeterministicRL, RecombiningPortfolioOptimisticNoisyDiscreteOnePlusOne], ["half"]
+# ).set_name("NoisyRL2", register=True)
+# NoisyRL3 = Chaining([MixDeterministicRL, OptimisticNoisyOnePlusOne], ["half"]).set_name(
+#    "NoisyRL3", register=True
+# )
+
+# High-Speed variants
+HSDE = DifferentialEvolution(high_speed=True).set_name("HSDE", register=True)
+LhsHSDE = DifferentialEvolution(initialization="LHS", high_speed=True).set_name("LhsHSDE", register=True)
