@@ -1,4 +1,4 @@
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -9,14 +9,25 @@ Caution
 This module and its available classes are experimental and may change quickly in the near future.
 """
 from .optimizerlib import ParametrizedOnePlusOne
-from .optimizerlib import EMNA
 from .optimizerlib import ParametrizedTBPSA
-from .optimizerlib import ParametrizedBO
+from .optimizerlib import ParametrizedMetaModel
+
+try:
+    from .optimizerlib import ParametrizedBO
+except ImportError:
+    pass  # bayes_opt not available
 from .optimizerlib import ParametrizedCMA
+from .optimizerlib import EMNA
 from .optimizerlib import Chaining
+from .optimizerlib import NoisySplit
+from .optimizerlib import ConfPortfolio
+from .optimizerlib import ConfPSO
+from .optimizerlib import ConfSplitOptimizer
+from .optimizerlib import BayesOptim
 from .differentialevolution import DifferentialEvolution
 from .es import EvolutionStrategy
-from .recastlib import ScipyOptimizer
+from .recastlib import NonObjectOptimizer
+from .recastlib import Pymoo
 from .oneshot import RandomSearchMaker
 from .oneshot import SamplingSearch
 
@@ -25,12 +36,19 @@ __all__ = [
     "ParametrizedOnePlusOne",
     "ParametrizedCMA",
     "ParametrizedBO",
+    "ParametrizedTBPSA",
+    "ParametrizedMetaModel",
     "DifferentialEvolution",
     "EvolutionStrategy",
-    "ScipyOptimizer",
+    "NonObjectOptimizer",
+    "Pymoo",
     "RandomSearchMaker",
     "SamplingSearch",
     "Chaining",
     "EMNA",
-    "ParametrizedTBPSA",
+    "NoisySplit",
+    "ConfPortfolio",
+    "ConfPSO",
+    "ConfSplitOptimizer",
+    "BayesOptim",
 ]
