@@ -1,13 +1,14 @@
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+import copy
 import numpy as np
-from scipy import stats
 from scipy.spatial import ConvexHull  # pylint: disable=no-name-in-module
 import nevergrad.common.typing as tp
 from nevergrad.parametrization import parameter as p
+from nevergrad.parametrization import transforms as trans
 from . import sequences
 from . import base
 from .base import IntOrParameter
@@ -25,17 +26,27 @@ def convex_limit(struct_points: np.ndarray) -> int:
         return len(struct_points) // 2
     for i in range(0, min(2 * d + 2, len(struct_points)), 2):
         points += [struct_points[i]]
-    hull = ConvexHull(points, incremental=True)
-    k = len(points)
-    for i in range(d + 1, len(points)):
+    hull = ConvexHull(points[: (d + 1)], incremental=True)
+    num_points = len(hull.vertices)
+    k = len(points) - 1
+    for i in range(num_points, len(points)):
+        # We add the ith point.
         hull.add_points(points[i : (i + 1)])
-        if i not in hull.vertices:
-            k = i - 1
-            break
+        num_points += 1
+        if len(hull.vertices) != num_points:
+            return num_points - 1
+        for j in range(i + 1, len(points)):
+            # We check that the jth point is not in the convex hull;
+            # this is ok if the jth point, added in the hull, becomes a vertex.
+            hull_copy = copy.deepcopy(hull)
+            hull_copy.add_points(points[j : j + 1])
+            if len(hull_copy.vertices) != num_points + 1:
+                return num_points - 1
     return k
 
 
 def hull_center(points: np.ndarray, k: int) -> np.ndarray:
+    """Center of the cuboid enclosing the hull."""
     hull = ConvexHull(points[:k])
     maxi = np.asarray(hull.vertices[0])
     mini = np.asarray(hull.vertices[0])
@@ -62,14 +73,14 @@ def avg_of_k_best(archive: utils.Archive[utils.MultiValue], method: str = "dimfo
     if method == "dimfourth":
         k = min(len(archive) // 4, dimension)  # fteytaud heuristic.
     elif method == "exp":
-        k = max(1, int(len(archive) // (1.1 ** dimension)))
+        k = max(1, int(len(archive) // (1.1**dimension)))
     elif method == "hull":
         k = convex_limit(
             np.concatenate(
                 sorted(items, key=lambda indiv: archive[indiv[0]].get_estimation("pessimistic")), axis=0
             )
         )
-        k = min(len(archive) // 4, min(k, int(len(archive) / (1.1 ** dimension))))
+        k = min(len(archive) // 4, min(k, int(len(archive) / (1.1**dimension))))
         # We might investigate the possibility to return the middle of the convex hull instead of averaging:
         # return hull_center(np.concatenate(sorted(items, key=lambda indiv: archive[indiv[0]].get_estimation("pessimistic")), axis=0), k)
     else:
@@ -123,7 +134,7 @@ class _RandomSearch(OneShotOptimizer):
         recommendation_rule: str = "pessimistic",
     ) -> None:
         super().__init__(parametrization, budget=budget, num_workers=num_workers)
-        assert opposition_mode is None or opposition_mode in ["quasi", "opposite"]
+        assert opposition_mode is None or opposition_mode in ["quasi", "opposite", "special"]
         assert isinstance(scale, (int, float)) or scale in ["auto", "random", "autotune"]
         self.middle_point = middle_point
         self.opposition_mode = opposition_mode
@@ -132,13 +143,18 @@ class _RandomSearch(OneShotOptimizer):
         self.scale = scale
         self.sampler = sampler
         self._opposable_data: tp.Optional[np.ndarray] = None
+        self._no_hypervolume = True
 
     def _internal_ask(self) -> tp.ArrayLike:
         # pylint: disable=not-callable
         mode = self.opposition_mode
         if self._opposable_data is not None and mode is not None:
             data = self._opposable_data
-            data *= -(self._rng.uniform(0.0, 1.0) if mode == "quasi" else 1.0)
+            data *= -(
+                self._rng.uniform(0.0, 1.0)
+                if mode == "quasi"
+                else (np.exp(-self._rng.uniform(0.0, 5.0)) if mode == "special" else 1.0)
+            )
             self._opposable_data = None
             return data
         if self.middle_point and not self._num_ask:
@@ -147,8 +163,10 @@ class _RandomSearch(OneShotOptimizer):
         scale = self.scale
         if isinstance(scale, str) and scale == "auto":
             # Some variants use a rescaling depending on the budget and the dimension (1st version).
+            assert self.budget is not None
             scale = (1 + np.log(self.budget)) / (4 * np.log(self.dimension))
         if isinstance(scale, str) and scale == "autotune":
+            assert self.budget is not None
             scale = np.sqrt(np.log(self.budget) / self.dimension)
         if isinstance(scale, str) and scale == "random":
             scale = np.exp(self._rng.normal(0.0, 1.0) - 2.0) / np.sqrt(self.dimension)
@@ -161,7 +179,7 @@ class _RandomSearch(OneShotOptimizer):
             point = self.parametrization.sample().get_standardized_data(reference=self.parametrization)
         else:
             raise ValueError("Unkwnown sampler {self.sampler}")
-        self._opposable_data = scale * point
+        self._opposable_data = scale * point  # type: ignore
         return self._opposable_data  # type: ignore
 
     def _internal_provide_recommendation(self) -> tp.Optional[tp.ArrayLike]:
@@ -261,7 +279,8 @@ class _SamplingSearch(OneShotOptimizer):
         self.rescaled = rescaled
         self.recommendation_rule = recommendation_rule
         # rescale to the bounds if both are provided
-        self._scaler = utils.BoundScaler(self.parametrization)
+        self._no_hypervolume = True
+        self._normalizer = p.helpers.Normalizer(self.parametrization)
 
     @property
     def sampler(self) -> sequences.Sampler:
@@ -274,7 +293,9 @@ class _SamplingSearch(OneShotOptimizer):
                 "Random": sequences.RandomSampler,
             }
             internal_budget = (
-                (budget + 1) // 2 if budget and (self.opposition_mode in ["quasi", "opposite"]) else budget
+                (budget + 1) // 2
+                if budget and (self.opposition_mode in ["quasi", "special", "opposite"])
+                else budget
             )
             self._sampler_instance = samplers[self._sampler](
                 self.dimension, internal_budget, scrambling=self.scrambled, random_state=self._rng
@@ -293,21 +314,31 @@ class _SamplingSearch(OneShotOptimizer):
         if self._opposable_data is not None and mode is not None:
             # weird mypy error, revealed as array, but not accepting substraction
             data = self._opposable_data
-            data *= -(self._rng.uniform(0.0, 1.0) if mode == "quasi" else 1.0)
+            # data *= -(self._rng.uniform(0.0, 1.0) if mode == "quasi" else 1.0)
+            data *= -(
+                self._rng.uniform(0.0, 1.0)
+                if mode == "quasi"
+                else (np.exp(-self._rng.uniform(0.0, 5.0)) if mode == "special" else 1.0)
+            )
             self._opposable_data = None
             return data
         sample = self.sampler()
         if self._rescaler is not None:
             sample = self._rescaler.apply(sample)
         if self.autorescale is True or self.autorescale == "auto":
+            assert self.budget is not None
             self.scale = (1 + np.log(self.budget)) / (4 * np.log(self.dimension))
         if self.autorescale == "autotune":
+            assert self.budget is not None
             self.scale = np.sqrt(np.log(self.budget) / self.dimension)
 
-        def transf(x: np.ndarray) -> np.ndarray:
-            return self.scale * (stats.cauchy.ppf if self.cauchy else stats.norm.ppf)(x)  # type: ignore
+        transf = trans.CumulativeDensity(
+            0, 1, scale=self.scale, density="cauchy" if self.cauchy else "gaussian"
+        )
+        # hack since scale is not defined before the first hack (TODO: refactor)
+        self._normalizer.unbounded_transform = transf
 
-        self._opposable_data = self._scaler.transform(sample, transf)
+        self._opposable_data = self._normalizer.backward(sample)
         assert self._opposable_data is not None
         return self._opposable_data
 
@@ -383,13 +414,20 @@ MetaRecentering = SamplingSearch(
 MetaTuneRecentering = SamplingSearch(
     cauchy=False, autorescale="autotune", sampler="Hammersley", scrambled=True
 ).set_name("MetaTuneRecentering", register=True)
-HAvgMetaRecentering = SamplingSearch(
+HullAvgMetaTuneRecentering = SamplingSearch(
+    cauchy=False,
+    autorescale="autotune",
+    sampler="Hammersley",
+    scrambled=True,
+    recommendation_rule="average_of_hull_best",
+).set_name("HullAvgMetaTuneRecentering", register=True)
+HullAvgMetaRecentering = SamplingSearch(
     cauchy=False,
     autorescale=True,
     sampler="Hammersley",
     scrambled=True,
     recommendation_rule="average_of_hull_best",
-).set_name("HAvgMetaRecentering", register=True)
+).set_name("HullAvgMetaRecentering", register=True)
 AvgMetaRecenteringNoHull = SamplingSearch(
     cauchy=False,
     autorescale=True,
