@@ -1,4 +1,4 @@
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -20,6 +20,10 @@ from ._layering import Level as Level
 P = tp.TypeVar("P", bound="Parameter")
 
 
+def default_congruence(x: tp.Any) -> tp.Any:
+    return x
+
+
 # pylint: disable=too-many-public-methods
 class Parameter(Layered):
     """Class providing the core functionality of a parameter, aka
@@ -44,10 +48,20 @@ class Parameter(Layered):
     def __init__(self) -> None:
         # Main features
         super().__init__()
-        self.uid = uuid.uuid4().hex
+        self.tabu_congruence: tp.Any = default_congruence
+        self.neural = False
+        self.has_constraints = False
+        self.enforce_determinism = False
+        self.real_world = False
+        self.hptuning = False
+        self.tabu_fails = 0
         self._subobjects = utils.Subobjects(
             self, base=Parameter, attribute="__dict__"
         )  # registers and apply functions too all (sub-)Parameter attributes
+        self.tabu_length = 0
+        self.tabu_set: tp.Set[tp.Any] = set()
+        self.tabu_list: tp.List[tp.Any] = []
+        self.tabu_index = 0
         self.parents_uids: tp.List[str] = []
         self.heritage: tp.Dict[tp.Hashable, tp.Any] = {"lineage": self.uid}  # passed through to children
         self.loss: tp.Optional[float] = None  # associated loss
@@ -62,10 +76,6 @@ class Parameter(Layered):
         self._meta: tp.Dict[tp.Hashable, tp.Any] = {}  # for anything algorithm related
         self.function = utils.FunctionInfo()
 
-    @property
-    def descriptors(self) -> utils.DeprecatedDescriptors:  # TODO remove
-        return utils.DeprecatedDescriptors(self)
-
     @property
     def losses(self) -> np.ndarray:
         """Possibly multiobjective losses which were told
@@ -114,6 +124,9 @@ class Parameter(Layered):
         """Mutate parameters of the instance, and then its value"""
         self._check_frozen()
         self._subobjects.apply("mutate")
+        self._layers[-1]._layered_mutate()
+
+    def _layered_mutate(self) -> None:
         self.set_standardized_data(self.random_state.normal(size=self.dimension))
 
     def sample(self: P) -> P:
@@ -141,9 +154,10 @@ class Parameter(Layered):
         """
         if not others:
             return
-        self.random_state  # pylint: disable=pointless-statement
+        self._check_frozen()
         assert all(isinstance(o, self.__class__) for o in others)
         self._subobjects.apply("recombine", *others)
+        self._layers[-1]._layered_recombine(*others)
 
     def get_standardized_data(self: P, *, reference: P) -> np.ndarray:
         """Get the standardized data representing the value of the instance as an array in the optimization space.
@@ -206,7 +220,7 @@ class Parameter(Layered):
         ), f"Expected {type(self)} but got {type(sent_reference)} as reference"
         self._check_frozen()
         del self.value  # remove all cached information
-        self._internal_set_standardized_data(np.array(data, copy=False), reference=sent_reference)
+        self._internal_set_standardized_data(np.asarray(data), reference=sent_reference)
         return self
 
     def _internal_set_standardized_data(  # pylint: disable=unused-argument
@@ -242,11 +256,20 @@ class Parameter(Layered):
             strings.append(str(self.value))
         return ":".join(strings)
 
+    def __bool__(self) -> bool:
+        raise RuntimeError("bool check is not allowed to avoid confusion")
+
     # %% Constraint management
-    def satisfies_constraints(self) -> bool:
+    def satisfies_constraints(self, ref: tp.Optional[P] = None, no_tabu: bool = False) -> bool:
         """Whether the instance satisfies the constraints added through
         the `register_cheap_constraint` method
 
+        Parameters
+        ----------
+        ref
+            parameter of the optimization algorithm, if we want to check the tabu list.
+        no_tabu
+            if we want to ignore the Tabu system.
         Returns
         -------
         bool
@@ -255,11 +278,40 @@ class Parameter(Layered):
         inside = self._subobjects.apply("satisfies_constraints")
         if not all(inside.values()):
             return False
+        val = self.value
+        if not no_tabu and ref is not None and ref.tabu_length > 0 and self.tabu_fails < 30:
+            tabu_val = self.tabu_congruence(val)
+            if isinstance(tabu_val, np.ndarray):
+                tabu_val = hash(tabu_val.tobytes())
+            if isinstance(tabu_val, dict):
+                keys = sorted(list(tabu_val.keys()))
+                tabu_val = str(keys) + str([tabu_val[k] for k in keys])
+            try:
+                if tabu_val in ref.tabu_set:
+                    self.tabu_fails += 1
+                    return False
+                else:
+                    ref.tabu_set.add(tabu_val)
+                    if len(ref.tabu_list) > ref.tabu_index:
+                        ref.tabu_set.remove(ref.tabu_list[ref.tabu_index])
+                        ref.tabu_list[ref.tabu_index] = tabu_val
+                    else:
+                        assert len(ref.tabu_list) == ref.tabu_index
+                        ref.tabu_list += [tabu_val]
+                    ref.tabu_list[ref.tabu_index] = tabu_val
+                    ref.tabu_index = (ref.tabu_index + 1) % ref.tabu_length
+            except RuntimeError as e:
+                raise RuntimeError(f"{tabu_val} has type {type(tabu_val)}, and this leads to {e}.")
         if not self._constraint_checkers:
             return True
-        val = self.value
         return all(utils.float_penalty(func(val)) <= 0 for func in self._constraint_checkers)
 
+    def specify_tabu_length(
+        self,
+        tabu_length: int,
+    ) -> None:
+        self.tabu_length = tabu_length
+
     def register_cheap_constraint(
         self,
         func: tp.Union[tp.Callable[[tp.Any], bool], tp.Callable[[tp.Any], float]],
@@ -305,16 +357,16 @@ class Parameter(Layered):
         """
         if self._random_state is None:
             # use the setter, to make sure the random state is propagated to the variables
-            seed = np.random.randint(2 ** 32, dtype=np.uint32)  # better way?
+            seed = np.random.randint(2**32, dtype=np.uint32)  # better way?
             self._set_random_state(np.random.RandomState(seed))
         assert self._random_state is not None
         return self._random_state
 
     @random_state.setter
-    def random_state(self, random_state: np.random.RandomState) -> None:
+    def random_state(self, random_state: tp.Optional[np.random.RandomState]) -> None:
         self._set_random_state(random_state)
 
-    def _set_random_state(self, random_state: np.random.RandomState) -> None:
+    def _set_random_state(self, random_state: tp.Optional[np.random.RandomState]) -> None:
         self._random_state = random_state
         self._subobjects.apply("_set_random_state", random_state)
 
@@ -351,6 +403,11 @@ class Parameter(Layered):
         child._frozen = False
         child._subobjects = self._subobjects.new(child)
         child._meta = {}
+        child.tabu_length = self.tabu_length
+        child.real_world = self.real_world
+        child.enforce_determinism = self.enforce_determinism
+        child.hptuning = self.hptuning
+        child.neural = self.neural
         child.parents_uids = list(self.parents_uids)
         child.heritage = dict(self.heritage)
         child.loss = None
@@ -394,12 +451,12 @@ class Parameter(Layered):
                 f"Cannot modify frozen Parameter {self.name}, please spawn a child and modify it instead"
                 "(optimizers freeze the parametrization and all asked and told candidates to avoid border effects)"
             )
+        # make sure the random state is initialized if we need to update it (aka if not frozen)
+        self.random_state  # pylint: disable=pointless-statement
         self._subobjects.apply("_check_frozen")
 
 
-# Basic types and helpers #
-
-
+# Basic types #
 class Constant(Parameter):
     """Parameter-like object for simplifying management of constant parameters:
     mutation/recombination do nothing, value cannot be changed, standardize data is an empty array,
