@@ -1,4 +1,4 @@
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -26,7 +26,11 @@ from .data import Array as Array
 from .data import Scalar as Scalar
 from .data import Log as Log
 from . import mutation
+from ._datalayers import Angles as Angles
 
 # choices
 from .choice import Choice as Choice
 from .choice import TransitionChoice as TransitionChoice
+
+# other
+from . import helpers as helpers
